# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.messaging.v1.service import ServiceList
from twilio.rest.messaging.v1.session import SessionList
from twilio.rest.messaging.v1.webhook import WebhookList


class V1(Version):

    def __init__(self, domain):
        """
        Initialize the V1 version of Messaging

        :returns: V1 version of Messaging
        :rtype: twilio.rest.messaging.v1.V1.V1
        """
        super(V1, self).__init__(domain)
        self.version = 'v1'
        self._services = None
        self._sessions = None
        self._webhooks = None

    @property
    def services(self):
        """
        :rtype: twilio.rest.messaging.v1.service.ServiceList
        """
        if self._services is None:
            self._services = ServiceList(self)
        return self._services

    @property
    def sessions(self):
        """
        :rtype: twilio.rest.messaging.v1.session.SessionList
        """
        if self._sessions is None:
            self._sessions = SessionList(self)
        return self._sessions

    @property
    def webhooks(self):
        """
        :rtype: twilio.rest.messaging.v1.webhook.WebhookList
        """
        if self._webhooks is None:
            self._webhooks = WebhookList(self)
        return self._webhooks

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Messaging.V1>'
